# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .report import Report as Report
from .profile import Profile as Profile
from .snapshot import Snapshot as Snapshot
from .profile_to_collection import ProfileToCollection as ProfileToCollection
from .created_fields_to_profile import CreatedFieldsToProfile as CreatedFieldsToProfile



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Union, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    NOT_GIVEN,
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
)
from ._utils import (
    is_given,
    get_async_library,
)
from ._version import __version__
from .resources import prod, fields, outlines, snapshots, reports_v3, collections, created_fields, prompt_templates
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import EvrimError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.bulk import bulk
from .resources.tags import tags
from .resources.blank import blank
from .resources.profiles import profiles
from .resources.templates import templates

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Evrim", "AsyncEvrim", "Client", "AsyncClient"]


class Evrim(SyncAPIClient):
    prod: prod.ProdResource
    blank: blank.BlankResource
    bulk: bulk.BulkResource
    collections: collections.CollectionsResource
    created_fields: created_fields.CreatedFieldsResource
    fields: fields.FieldsResource
    outlines: outlines.OutlinesResource
    profiles: profiles.ProfilesResource
    prompt_templates: prompt_templates.PromptTemplatesResource
    reports_v3: reports_v3.ReportsV3Resource
    snapshots: snapshots.SnapshotsResource
    tags: tags.TagsResource
    templates: templates.TemplatesResource
    with_raw_response: EvrimWithRawResponse
    with_streaming_response: EvrimWithStreamedResponse

    # client options
    api_token: str

    def __init__(
        self,
        *,
        api_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous evrim client instance.

        This automatically infers the `api_token` argument from the `EVRIM_API_TOKEN` environment variable if it is not provided.
        """
        if api_token is None:
            api_token = os.environ.get("EVRIM_API_TOKEN")
        if api_token is None:
            raise EvrimError(
                "The api_token client option must be set either by passing api_token to the client or by setting the EVRIM_API_TOKEN environment variable"
            )
        self.api_token = api_token

        if base_url is None:
            base_url = os.environ.get("EVRIM_BASE_URL")
        if base_url is None:
            base_url = f"https://api.evrim.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.prod = prod.ProdResource(self)
        self.blank = blank.BlankResource(self)
        self.bulk = bulk.BulkResource(self)
        self.collections = collections.CollectionsResource(self)
        self.created_fields = created_fields.CreatedFieldsResource(self)
        self.fields = fields.FieldsResource(self)
        self.outlines = outlines.OutlinesResource(self)
        self.profiles = profiles.ProfilesResource(self)
        self.prompt_templates = prompt_templates.PromptTemplatesResource(self)
        self.reports_v3 = reports_v3.ReportsV3Resource(self)
        self.snapshots = snapshots.SnapshotsResource(self)
        self.tags = tags.TagsResource(self)
        self.templates = templates.TemplatesResource(self)
        self.with_raw_response = EvrimWithRawResponse(self)
        self.with_streaming_response = EvrimWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_token = self.api_token
        return {"Authorization": f"Bearer {api_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_token=api_token or self.api_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncEvrim(AsyncAPIClient):
    prod: prod.AsyncProdResource
    blank: blank.AsyncBlankResource
    bulk: bulk.AsyncBulkResource
    collections: collections.AsyncCollectionsResource
    created_fields: created_fields.AsyncCreatedFieldsResource
    fields: fields.AsyncFieldsResource
    outlines: outlines.AsyncOutlinesResource
    profiles: profiles.AsyncProfilesResource
    prompt_templates: prompt_templates.AsyncPromptTemplatesResource
    reports_v3: reports_v3.AsyncReportsV3Resource
    snapshots: snapshots.AsyncSnapshotsResource
    tags: tags.AsyncTagsResource
    templates: templates.AsyncTemplatesResource
    with_raw_response: AsyncEvrimWithRawResponse
    with_streaming_response: AsyncEvrimWithStreamedResponse

    # client options
    api_token: str

    def __init__(
        self,
        *,
        api_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: Union[float, Timeout, None, NotGiven] = NOT_GIVEN,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async evrim client instance.

        This automatically infers the `api_token` argument from the `EVRIM_API_TOKEN` environment variable if it is not provided.
        """
        if api_token is None:
            api_token = os.environ.get("EVRIM_API_TOKEN")
        if api_token is None:
            raise EvrimError(
                "The api_token client option must be set either by passing api_token to the client or by setting the EVRIM_API_TOKEN environment variable"
            )
        self.api_token = api_token

        if base_url is None:
            base_url = os.environ.get("EVRIM_BASE_URL")
        if base_url is None:
            base_url = f"https://api.evrim.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.prod = prod.AsyncProdResource(self)
        self.blank = blank.AsyncBlankResource(self)
        self.bulk = bulk.AsyncBulkResource(self)
        self.collections = collections.AsyncCollectionsResource(self)
        self.created_fields = created_fields.AsyncCreatedFieldsResource(self)
        self.fields = fields.AsyncFieldsResource(self)
        self.outlines = outlines.AsyncOutlinesResource(self)
        self.profiles = profiles.AsyncProfilesResource(self)
        self.prompt_templates = prompt_templates.AsyncPromptTemplatesResource(self)
        self.reports_v3 = reports_v3.AsyncReportsV3Resource(self)
        self.snapshots = snapshots.AsyncSnapshotsResource(self)
        self.tags = tags.AsyncTagsResource(self)
        self.templates = templates.AsyncTemplatesResource(self)
        self.with_raw_response = AsyncEvrimWithRawResponse(self)
        self.with_streaming_response = AsyncEvrimWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_token = self.api_token
        return {"Authorization": f"Bearer {api_token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_token: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = NOT_GIVEN,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = NOT_GIVEN,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_token=api_token or self.api_token,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class EvrimWithRawResponse:
    def __init__(self, client: Evrim) -> None:
        self.prod = prod.ProdResourceWithRawResponse(client.prod)
        self.blank = blank.BlankResourceWithRawResponse(client.blank)
        self.bulk = bulk.BulkResourceWithRawResponse(client.bulk)
        self.collections = collections.CollectionsResourceWithRawResponse(client.collections)
        self.created_fields = created_fields.CreatedFieldsResourceWithRawResponse(client.created_fields)
        self.fields = fields.FieldsResourceWithRawResponse(client.fields)
        self.outlines = outlines.OutlinesResourceWithRawResponse(client.outlines)
        self.profiles = profiles.ProfilesResourceWithRawResponse(client.profiles)
        self.prompt_templates = prompt_templates.PromptTemplatesResourceWithRawResponse(client.prompt_templates)
        self.reports_v3 = reports_v3.ReportsV3ResourceWithRawResponse(client.reports_v3)
        self.snapshots = snapshots.SnapshotsResourceWithRawResponse(client.snapshots)
        self.tags = tags.TagsResourceWithRawResponse(client.tags)
        self.templates = templates.TemplatesResourceWithRawResponse(client.templates)


class AsyncEvrimWithRawResponse:
    def __init__(self, client: AsyncEvrim) -> None:
        self.prod = prod.AsyncProdResourceWithRawResponse(client.prod)
        self.blank = blank.AsyncBlankResourceWithRawResponse(client.blank)
        self.bulk = bulk.AsyncBulkResourceWithRawResponse(client.bulk)
        self.collections = collections.AsyncCollectionsResourceWithRawResponse(client.collections)
        self.created_fields = created_fields.AsyncCreatedFieldsResourceWithRawResponse(client.created_fields)
        self.fields = fields.AsyncFieldsResourceWithRawResponse(client.fields)
        self.outlines = outlines.AsyncOutlinesResourceWithRawResponse(client.outlines)
        self.profiles = profiles.AsyncProfilesResourceWithRawResponse(client.profiles)
        self.prompt_templates = prompt_templates.AsyncPromptTemplatesResourceWithRawResponse(client.prompt_templates)
        self.reports_v3 = reports_v3.AsyncReportsV3ResourceWithRawResponse(client.reports_v3)
        self.snapshots = snapshots.AsyncSnapshotsResourceWithRawResponse(client.snapshots)
        self.tags = tags.AsyncTagsResourceWithRawResponse(client.tags)
        self.templates = templates.AsyncTemplatesResourceWithRawResponse(client.templates)


class EvrimWithStreamedResponse:
    def __init__(self, client: Evrim) -> None:
        self.prod = prod.ProdResourceWithStreamingResponse(client.prod)
        self.blank = blank.BlankResourceWithStreamingResponse(client.blank)
        self.bulk = bulk.BulkResourceWithStreamingResponse(client.bulk)
        self.collections = collections.CollectionsResourceWithStreamingResponse(client.collections)
        self.created_fields = created_fields.CreatedFieldsResourceWithStreamingResponse(client.created_fields)
        self.fields = fields.FieldsResourceWithStreamingResponse(client.fields)
        self.outlines = outlines.OutlinesResourceWithStreamingResponse(client.outlines)
        self.profiles = profiles.ProfilesResourceWithStreamingResponse(client.profiles)
        self.prompt_templates = prompt_templates.PromptTemplatesResourceWithStreamingResponse(client.prompt_templates)
        self.reports_v3 = reports_v3.ReportsV3ResourceWithStreamingResponse(client.reports_v3)
        self.snapshots = snapshots.SnapshotsResourceWithStreamingResponse(client.snapshots)
        self.tags = tags.TagsResourceWithStreamingResponse(client.tags)
        self.templates = templates.TemplatesResourceWithStreamingResponse(client.templates)


class AsyncEvrimWithStreamedResponse:
    def __init__(self, client: AsyncEvrim) -> None:
        self.prod = prod.AsyncProdResourceWithStreamingResponse(client.prod)
        self.blank = blank.AsyncBlankResourceWithStreamingResponse(client.blank)
        self.bulk = bulk.AsyncBulkResourceWithStreamingResponse(client.bulk)
        self.collections = collections.AsyncCollectionsResourceWithStreamingResponse(client.collections)
        self.created_fields = created_fields.AsyncCreatedFieldsResourceWithStreamingResponse(client.created_fields)
        self.fields = fields.AsyncFieldsResourceWithStreamingResponse(client.fields)
        self.outlines = outlines.AsyncOutlinesResourceWithStreamingResponse(client.outlines)
        self.profiles = profiles.AsyncProfilesResourceWithStreamingResponse(client.profiles)
        self.prompt_templates = prompt_templates.AsyncPromptTemplatesResourceWithStreamingResponse(
            client.prompt_templates
        )
        self.reports_v3 = reports_v3.AsyncReportsV3ResourceWithStreamingResponse(client.reports_v3)
        self.snapshots = snapshots.AsyncSnapshotsResourceWithStreamingResponse(client.snapshots)
        self.tags = tags.AsyncTagsResourceWithStreamingResponse(client.tags)
        self.templates = templates.AsyncTemplatesResourceWithStreamingResponse(client.templates)


Client = Evrim

AsyncClient = AsyncEvrim



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["CollectionUpdateParams"]


class CollectionUpdateParams(TypedDict, total=False):
    description: str

    name: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ReportsV3UpdateParams"]


class ReportsV3UpdateParams(TypedDict, total=False):
    outline_id: int

    profile_id: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import prompt_template_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.prompt_template import PromptTemplate
from ..types.prompt_template_list_response import PromptTemplateListResponse

__all__ = ["PromptTemplatesResource", "AsyncPromptTemplatesResource"]


class PromptTemplatesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PromptTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/evrimai/python-client#accessing-raw-response-data-eg-headers
        """
        return PromptTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PromptTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/evrimai/python-client#with_streaming_response
        """
        return PromptTemplatesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        prompt: str,
        status: Literal["W", "P", "C", "F"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PromptTemplate:
        """
        Args:
          status: - `W` - WAITING
              - `P` - PROCESSING
              - `C` - COMPLETED
              - `F` - FAILED

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/prod/v0/prompt-templates/",
            body=maybe_transform(
                {
                    "prompt": prompt,
                    "status": status,
                },
                prompt_template_create_params.PromptTemplateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromptTemplate,
        )

    def retrieve(
        self,
        id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PromptTemplate:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            f"/prod/v0/prompt-templates/{id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromptTemplate,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PromptTemplateListResponse:
        return self._get(
            "/prod/v0/prompt-templates/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromptTemplateListResponse,
        )


class AsyncPromptTemplatesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPromptTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/evrimai/python-client#accessing-raw-response-data-eg-headers
        """
        return AsyncPromptTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPromptTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/evrimai/python-client#with_streaming_response
        """
        return AsyncPromptTemplatesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        prompt: str,
        status: Literal["W", "P", "C", "F"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PromptTemplate:
        """
        Args:
          status: - `W` - WAITING
              - `P` - PROCESSING
              - `C` - COMPLETED
              - `F` - FAILED

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/prod/v0/prompt-templates/",
            body=await async_maybe_transform(
                {
                    "prompt": prompt,
                    "status": status,
                },
                prompt_template_create_params.PromptTemplateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromptTemplate,
        )

    async def retrieve(
        self,
        id: int,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PromptTemplate:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            f"/prod/v0/prompt-templates/{id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromptTemplate,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PromptTemplateListResponse:
        return await self._get(
            "/prod/v0/prompt-templates/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PromptTemplateListResponse,
        )


class PromptTemplatesResourceWithRawResponse:
    def __init__(self, prompt_templates: PromptTemplatesResource) -> None:
        self._prompt_templates = prompt_templates

        self.create = to_raw_response_wrapper(
            prompt_templates.create,
        )
        self.retrieve = to_raw_response_wrapper(
            prompt_templates.retrieve,
        )
        self.list = to_raw_response_wrapper(
            prompt_templates.list,
        )


class AsyncPromptTemplatesResourceWithRawResponse:
    def __init__(self, prompt_templates: AsyncPromptTemplatesResource) -> None:
        self._prompt_templates = prompt_templates

        self.create = async_to_raw_response_wrapper(
            prompt_templates.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            prompt_templates.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            prompt_templates.list,
        )


class PromptTemplatesResourceWithStreamingResponse:
    def __init__(self, prompt_templates: PromptTemplatesResource) -> None:
        self._prompt_templates = prompt_templates

        self.create = to_streamed_response_wrapper(
            prompt_templates.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            prompt_templates.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            prompt_templates.list,
        )


class AsyncPromptTemplatesResourceWithStreamingResponse:
    def __init__(self, prompt_templates: AsyncPromptTemplatesResource) -> None:
        self._prompt_templates = prompt_templates

        self.create = async_to_streamed_response_wrapper(
            prompt_templates.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            prompt_templates.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            prompt_templates.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ..shared.report import Report

__all__ = ["ReportListResponse"]

ReportListResponse: TypeAlias = List[Report]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .prompt_template import PromptTemplate

__all__ = ["PromptTemplateListResponse"]

PromptTemplateListResponse: TypeAlias = List[PromptTemplate]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from ...template import Template
from ...shared.profile import Profile

__all__ = ["BulkJob"]


class BulkJob(BaseModel):
    id: int

    error: str

    profiles: List[Profile]

    status: str

    template: Template



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.profiles.snapshot_list_response import SnapshotListResponse
from ...types.profiles.create_profile_snapshot import CreateProfileSnapshot

__all__ = ["SnapshotsResource", "AsyncSnapshotsResource"]


class SnapshotsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SnapshotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/evrimai/python-client#accessing-raw-response-data-eg-headers
        """
        return SnapshotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SnapshotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/evrimai/python-client#with_streaming_response
        """
        return SnapshotsResourceWithStreamingResponse(self)

    def create(
        self,
        profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateProfileSnapshot:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return self._post(
            f"/prod/v0/profiles/{profile_id}/snapshots/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateProfileSnapshot,
        )

    def retrieve(
        self,
        snapshot_id: str,
        *,
        profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateProfileSnapshot:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        if not snapshot_id:
            raise ValueError(f"Expected a non-empty value for `snapshot_id` but received {snapshot_id!r}")
        return self._get(
            f"/prod/v0/profiles/{profile_id}/snapshots/{snapshot_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateProfileSnapshot,
        )

    def list(
        self,
        profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SnapshotListResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return self._get(
            f"/prod/v0/profiles/{profile_id}/snapshots/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SnapshotListResponse,
        )


class AsyncSnapshotsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSnapshotsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/evrimai/python-client#accessing-raw-response-data-eg-headers
        """
        return AsyncSnapshotsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSnapshotsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/evrimai/python-client#with_streaming_response
        """
        return AsyncSnapshotsResourceWithStreamingResponse(self)

    async def create(
        self,
        profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateProfileSnapshot:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return await self._post(
            f"/prod/v0/profiles/{profile_id}/snapshots/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateProfileSnapshot,
        )

    async def retrieve(
        self,
        snapshot_id: str,
        *,
        profile_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateProfileSnapshot:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        if not snapshot_id:
            raise ValueError(f"Expected a non-empty value for `snapshot_id` but received {snapshot_id!r}")
        return await self._get(
            f"/prod/v0/profiles/{profile_id}/snapshots/{snapshot_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateProfileSnapshot,
        )

    async def list(
        self,
        profile_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SnapshotListResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not profile_id:
            raise ValueError(f"Expected a non-empty value for `profile_id` but received {profile_id!r}")
        return await self._get(
            f"/prod/v0/profiles/{profile_id}/snapshots/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SnapshotListResponse,
        )


class SnapshotsResourceWithRawResponse:
    def __init__(self, snapshots: SnapshotsResource) -> None:
        self._snapshots = snapshots

        self.create = to_raw_response_wrapper(
            snapshots.create,
        )
        self.retrieve = to_raw_response_wrapper(
            snapshots.retrieve,
        )
        self.list = to_raw_response_wrapper(
            snapshots.list,
        )


class AsyncSnapshotsResourceWithRawResponse:
    def __init__(self, snapshots: AsyncSnapshotsResource) -> None:
        self._snapshots = snapshots

        self.create = async_to_raw_response_wrapper(
            snapshots.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            snapshots.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            snapshots.list,
        )


class SnapshotsResourceWithStreamingResponse:
    def __init__(self, snapshots: SnapshotsResource) -> None:
        self._snapshots = snapshots

        self.create = to_streamed_response_wrapper(
            snapshots.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            snapshots.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            snapshots.list,
        )


class AsyncSnapshotsResourceWithStreamingResponse:
    def __init__(self, snapshots: AsyncSnapshotsResource) -> None:
        self._snapshots = snapshots

        self.create = async_to_streamed_response_wrapper(
            snapshots.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            snapshots.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            snapshots.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["ProfileTagParams"]


class ProfileTagParams(TypedDict, total=False):
    profile_ids: Required[Iterable[int]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["Field"]


class Field(BaseModel):
    description: str

    name: str

    type: str

    id: Optional[int] = None

    enum_many: Optional[bool] = None

    enum_values: Optional[List[str]] = None

    rel_template: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from evrim import Evrim, AsyncEvrim
from tests.utils import assert_matches_type
from evrim.types.tags import TagToCollection

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCollections:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_tag(self, client: Evrim) -> None:
        collection = client.tags.collections.tag(
            tag_id="321669910225",
            collection_id=0,
        )
        assert_matches_type(TagToCollection, collection, path=["response"])

    @parametrize
    def test_raw_response_tag(self, client: Evrim) -> None:
        response = client.tags.collections.with_raw_response.tag(
            tag_id="321669910225",
            collection_id=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        collection = response.parse()
        assert_matches_type(TagToCollection, collection, path=["response"])

    @parametrize
    def test_streaming_response_tag(self, client: Evrim) -> None:
        with client.tags.collections.with_streaming_response.tag(
            tag_id="321669910225",
            collection_id=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            collection = response.parse()
            assert_matches_type(TagToCollection, collection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_tag(self, client: Evrim) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tag_id` but received ''"):
            client.tags.collections.with_raw_response.tag(
                tag_id="",
                collection_id=0,
            )


class TestAsyncCollections:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_tag(self, async_client: AsyncEvrim) -> None:
        collection = await async_client.tags.collections.tag(
            tag_id="321669910225",
            collection_id=0,
        )
        assert_matches_type(TagToCollection, collection, path=["response"])

    @parametrize
    async def test_raw_response_tag(self, async_client: AsyncEvrim) -> None:
        response = await async_client.tags.collections.with_raw_response.tag(
            tag_id="321669910225",
            collection_id=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        collection = await response.parse()
        assert_matches_type(TagToCollection, collection, path=["response"])

    @parametrize
    async def test_streaming_response_tag(self, async_client: AsyncEvrim) -> None:
        async with async_client.tags.collections.with_streaming_response.tag(
            tag_id="321669910225",
            collection_id=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            collection = await response.parse()
            assert_matches_type(TagToCollection, collection, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_tag(self, async_client: AsyncEvrim) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tag_id` but received ''"):
            await async_client.tags.collections.with_raw_response.tag(
                tag_id="",
                collection_id=0,
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .profile_create_params import ProfileCreateParams as ProfileCreateParams
from .bulk_profiles_to_collection import BulkProfilesToCollection as BulkProfilesToCollection



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tag import Tag as Tag
from .field import Field as Field
from .shared import (
    Report as Report,
    Profile as Profile,
    Snapshot as Snapshot,
    ProfileToCollection as ProfileToCollection,
    CreatedFieldsToProfile as CreatedFieldsToProfile,
)
from .outline import Outline as Outline
from .template import Template as Template
from .tag_param import TagParam as TagParam
from .collection import Collection as Collection
from .field_param import FieldParam as FieldParam
from .tag_profile import TagProfile as TagProfile
from .created_field import CreatedField as CreatedField
from .template_param import TemplateParam as TemplateParam
from .prompt_template import PromptTemplate as PromptTemplate
from .field_to_template import FieldToTemplate as FieldToTemplate
from .tag_create_params import TagCreateParams as TagCreateParams
from .tag_list_response import TagListResponse as TagListResponse
from .tag_update_params import TagUpdateParams as TagUpdateParams
from .prod_schema_params import ProdSchemaParams as ProdSchemaParams
from .profile_tag_params import ProfileTagParams as ProfileTagParams
from .field_create_params import FieldCreateParams as FieldCreateParams
from .field_list_response import FieldListResponse as FieldListResponse
from .field_update_params import FieldUpdateParams as FieldUpdateParams
from .prod_schema_response import ProdSchemaResponse as ProdSchemaResponse
from .field_template_params import FieldTemplateParams as FieldTemplateParams
from .outline_create_params import OutlineCreateParams as OutlineCreateParams
from .outline_list_response import OutlineListResponse as OutlineListResponse
from .outline_update_params import OutlineUpdateParams as OutlineUpdateParams
from .profile_create_params import ProfileCreateParams as ProfileCreateParams
from .profile_list_response import ProfileListResponse as ProfileListResponse
from .profile_update_params import ProfileUpdateParams as ProfileUpdateParams
from .snapshot_list_response import SnapshotListResponse as SnapshotListResponse
from .template_create_params import TemplateCreateParams as TemplateCreateParams
from .template_list_response import TemplateListResponse as TemplateListResponse
from .template_update_params import TemplateUpdateParams as TemplateUpdateParams
from .collection_create_params import CollectionCreateParams as CollectionCreateParams
from .collection_list_response import CollectionListResponse as CollectionListResponse
from .collection_update_params import CollectionUpdateParams as CollectionUpdateParams
from .reports_v3_create_params import ReportsV3CreateParams as ReportsV3CreateParams
from .reports_v3_list_response import ReportsV3ListResponse as ReportsV3ListResponse
from .reports_v3_update_params import ReportsV3UpdateParams as ReportsV3UpdateParams
from .created_field_create_params import CreatedFieldCreateParams as CreatedFieldCreateParams
from .created_field_list_response import CreatedFieldListResponse as CreatedFieldListResponse
from .created_field_update_params import CreatedFieldUpdateParams as CreatedFieldUpdateParams
from .created_field_profile_params import CreatedFieldProfileParams as CreatedFieldProfileParams
from .prompt_template_create_params import PromptTemplateCreateParams as PromptTemplateCreateParams
from .prompt_template_list_response import PromptTemplateListResponse as PromptTemplateListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["PromptTemplateCreateParams"]


class PromptTemplateCreateParams(TypedDict, total=False):
    prompt: Required[str]

    status: Literal["W", "P", "C", "F"]
    """
    - `W` - WAITING
    - `P` - PROCESSING
    - `C` - COMPLETED
    - `F` - FAILED
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from .field import Field
from .._models import BaseModel

__all__ = ["CreatedField"]


class CreatedField(BaseModel):
    id: int

    field: Field

    value: object



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel
from .template import Template

__all__ = ["PromptTemplate"]


class PromptTemplate(BaseModel):
    id: int

    prompt: str

    template: Template

    status: Optional[Literal["W", "P", "C", "F"]] = None
    """
    - `W` - WAITING
    - `P` - PROCESSING
    - `C` - COMPLETED
    - `F` - FAILED
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.tags import profile_tag_params
from ..._base_client import make_request_options
from ...types.tags.profile_list_response import ProfileListResponse

__all__ = ["ProfilesResource", "AsyncProfilesResource"]


class ProfilesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/evrimai/python-client#accessing-raw-response-data-eg-headers
        """
        return ProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/evrimai/python-client#with_streaming_response
        """
        return ProfilesResourceWithStreamingResponse(self)

    def list(
        self,
        tag_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProfileListResponse:
        """
        Get all profiles associated with a tag

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        return self._get(
            f"/prod/v0/tags/{tag_id}/profiles/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProfileListResponse,
        )

    def tag(
        self,
        tag_id: str,
        *,
        profile_ids: Iterable[int],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Tag all profiles with the tag

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/prod/v0/tags/{tag_id}/profiles/",
            body=maybe_transform({"profile_ids": profile_ids}, profile_tag_params.ProfileTagParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncProfilesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/evrimai/python-client#accessing-raw-response-data-eg-headers
        """
        return AsyncProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/evrimai/python-client#with_streaming_response
        """
        return AsyncProfilesResourceWithStreamingResponse(self)

    async def list(
        self,
        tag_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProfileListResponse:
        """
        Get all profiles associated with a tag

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        return await self._get(
            f"/prod/v0/tags/{tag_id}/profiles/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProfileListResponse,
        )

    async def tag(
        self,
        tag_id: str,
        *,
        profile_ids: Iterable[int],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Tag all profiles with the tag

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tag_id:
            raise ValueError(f"Expected a non-empty value for `tag_id` but received {tag_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/prod/v0/tags/{tag_id}/profiles/",
            body=await async_maybe_transform({"profile_ids": profile_ids}, profile_tag_params.ProfileTagParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ProfilesResourceWithRawResponse:
    def __init__(self, profiles: ProfilesResource) -> None:
        self._profiles = profiles

        self.list = to_raw_response_wrapper(
            profiles.list,
        )
        self.tag = to_raw_response_wrapper(
            profiles.tag,
        )


class AsyncProfilesResourceWithRawResponse:
    def __init__(self, profiles: AsyncProfilesResource) -> None:
        self._profiles = profiles

        self.list = async_to_raw_response_wrapper(
            profiles.list,
        )
        self.tag = async_to_raw_response_wrapper(
            profiles.tag,
        )


class ProfilesResourceWithStreamingResponse:
    def __init__(self, profiles: ProfilesResource) -> None:
        self._profiles = profiles

        self.list = to_streamed_response_wrapper(
            profiles.list,
        )
        self.tag = to_streamed_response_wrapper(
            profiles.tag,
        )


class AsyncProfilesResourceWithStreamingResponse:
    def __init__(self, profiles: AsyncProfilesResource) -> None:
        self._profiles = profiles

        self.list = async_to_streamed_response_wrapper(
            profiles.list,
        )
        self.tag = async_to_streamed_response_wrapper(
            profiles.tag,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .outline import Outline

__all__ = ["OutlineListResponse"]

OutlineListResponse: TypeAlias = List[Outline]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from ..template_param import TemplateParam

__all__ = ["CreatedFieldCreateParams", "CreateField"]


class CreatedFieldCreateParams(TypedDict, total=False):
    create_fields: Required[Iterable[CreateField]]

    specification: Required[str]


class CreateField(TypedDict, total=False):
    description: Required[str]

    name: Required[str]

    type: Required[str]

    rel_template: TemplateParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .bulk import (
    BulkResource,
    AsyncBulkResource,
    BulkResourceWithRawResponse,
    AsyncBulkResourceWithRawResponse,
    BulkResourceWithStreamingResponse,
    AsyncBulkResourceWithStreamingResponse,
)
from .templates import (
    TemplatesResource,
    AsyncTemplatesResource,
    TemplatesResourceWithRawResponse,
    AsyncTemplatesResourceWithRawResponse,
    TemplatesResourceWithStreamingResponse,
    AsyncTemplatesResourceWithStreamingResponse,
)
from .collections import (
    CollectionsResource,
    AsyncCollectionsResource,
    CollectionsResourceWithRawResponse,
    AsyncCollectionsResourceWithRawResponse,
    CollectionsResourceWithStreamingResponse,
    AsyncCollectionsResourceWithStreamingResponse,
)
from .created_fields import (
    CreatedFieldsResource,
    AsyncCreatedFieldsResource,
    CreatedFieldsResourceWithRawResponse,
    AsyncCreatedFieldsResourceWithRawResponse,
    CreatedFieldsResourceWithStreamingResponse,
    AsyncCreatedFieldsResourceWithStreamingResponse,
)

__all__ = [
    "CollectionsResource",
    "AsyncCollectionsResource",
    "CollectionsResourceWithRawResponse",
    "AsyncCollectionsResourceWithRawResponse",
    "CollectionsResourceWithStreamingResponse",
    "AsyncCollectionsResourceWithStreamingResponse",
    "CreatedFieldsResource",
    "AsyncCreatedFieldsResource",
    "CreatedFieldsResourceWithRawResponse",
    "AsyncCreatedFieldsResourceWithRawResponse",
    "CreatedFieldsResourceWithStreamingResponse",
    "AsyncCreatedFieldsResourceWithStreamingResponse",
    "TemplatesResource",
    "AsyncTemplatesResource",
    "TemplatesResourceWithRawResponse",
    "AsyncTemplatesResourceWithRawResponse",
    "TemplatesResourceWithStreamingResponse",
    "AsyncTemplatesResourceWithStreamingResponse",
    "BulkResource",
    "AsyncBulkResource",
    "BulkResourceWithRawResponse",
    "AsyncBulkResourceWithRawResponse",
    "BulkResourceWithStreamingResponse",
    "AsyncBulkResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .field import Field
from .._models import BaseModel

__all__ = ["Template"]


class Template(BaseModel):
    id: int

    fields: List[Field]

    name: str

    description: Optional[str] = None

    questions: Optional[List[str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ReportsV3CreateParams"]


class ReportsV3CreateParams(TypedDict, total=False):
    outline_id: Required[int]

    profile_id: Required[int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tags import (
    TagsResource,
    AsyncTagsResource,
    TagsResourceWithRawResponse,
    AsyncTagsResourceWithRawResponse,
    TagsResourceWithStreamingResponse,
    AsyncTagsResourceWithStreamingResponse,
)
from .profiles import (
    ProfilesResource,
    AsyncProfilesResource,
    ProfilesResourceWithRawResponse,
    AsyncProfilesResourceWithRawResponse,
    ProfilesResourceWithStreamingResponse,
    AsyncProfilesResourceWithStreamingResponse,
)
from .collections import (
    CollectionsResource,
    AsyncCollectionsResource,
    CollectionsResourceWithRawResponse,
    AsyncCollectionsResourceWithRawResponse,
    CollectionsResourceWithStreamingResponse,
    AsyncCollectionsResourceWithStreamingResponse,
)

__all__ = [
    "CollectionsResource",
    "AsyncCollectionsResource",
    "CollectionsResourceWithRawResponse",
    "AsyncCollectionsResourceWithRawResponse",
    "CollectionsResourceWithStreamingResponse",
    "AsyncCollectionsResourceWithStreamingResponse",
    "ProfilesResource",
    "AsyncProfilesResource",
    "ProfilesResourceWithRawResponse",
    "AsyncProfilesResourceWithRawResponse",
    "ProfilesResourceWithStreamingResponse",
    "AsyncProfilesResourceWithStreamingResponse",
    "TagsResource",
    "AsyncTagsResource",
    "TagsResourceWithRawResponse",
    "AsyncTagsResourceWithRawResponse",
    "TagsResourceWithStreamingResponse",
    "AsyncTagsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from ...._models import BaseModel

__all__ = ["BulkProfilesToCollection"]


class BulkProfilesToCollection(BaseModel):
    pass



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .report_list_response import ReportListResponse as ReportListResponse
from .snapshot_list_response import SnapshotListResponse as SnapshotListResponse
from .create_profile_snapshot import CreateProfileSnapshot as CreateProfileSnapshot
from .collection_create_params import CollectionCreateParams as CollectionCreateParams
from .latest_retrieve_response import LatestRetrieveResponse as LatestRetrieveResponse
from .created_field_create_params import CreatedFieldCreateParams as CreatedFieldCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .field import Field

__all__ = ["FieldListResponse"]

FieldListResponse: TypeAlias = List[Field]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["Outline"]


class Outline(BaseModel):
    id: int

    perspective: str

    outline: Optional[object] = None
